from typing import Any

import pulse as ps


@ps.react_component("Tabs", "@mantine/core")
def Tabs(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Tabs", "@mantine/core", prop="Tab")
def TabsTab(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Tabs", "@mantine/core", prop="Panel")
def TabsPanel(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Tabs", "@mantine/core", prop="List")
def TabsList(*children: ps.Child, key: str | None = None, **props: Any): ...



from __future__ import annotations

# Auto-generated by scripts/generate_public_api.py. Do not edit manually.
from .charts.area_chart import (
	AreaChart,
)
from .charts.bar_chart import (
	BarChart,
)
from .charts.bubble_chart import (
	BubbleChart,
)
from .charts.chart_legend import (
	ChartLegend,
)
from .charts.chart_tooltip import (
	ChartTooltip,
)
from .charts.composite_chart import (
	CompositeChart,
)
from .charts.donut_chart import (
	DonutChart,
)
from .charts.funnel_chart import (
	FunnelChart,
)
from .charts.heatmap import (
	Heatmap,
)
from .charts.line_chart import (
	LineChart,
)
from .charts.pie_chart import (
	PieChart,
)
from .charts.radar_chart import (
	RadarChart,
)
from .charts.radial_bar_chart import (
	RadialBarChart,
)
from .charts.scatter_chart import (
	ScatterChart,
)
from .charts.sparkline import (
	Sparkline,
)
from .core.base import (
	MantineComponentProps,
)
from .core.box import (
	Box,
	BoxComponentProps,
	BoxMod,
	BoxProps,
	Mod,
)
from .core.buttons.action_icon import (
	ActionIcon,
	ActionIconGroup,
	ActionIconGroupSection,
)
from .core.buttons.button import (
	Button,
	ButtonGroup,
	ButtonGroupSection,
)
from .core.buttons.close_button import (
	CloseButton,
)
from .core.buttons.copy_button import (
	CopyButton,
)
from .core.buttons.file_button import (
	FileButton,
)
from .core.buttons.unstyled_button import (
	UnstyledButton,
)
from .core.combobox.autocomplete import (
	Autocomplete,
)
from .core.combobox.combobox import (
	Combobox,
	ComboboxChevron,
	ComboboxClearButton,
	ComboboxDropdown,
	ComboboxDropdownTarget,
	ComboboxEmpty,
	ComboboxEventsTarget,
	ComboboxFooter,
	ComboboxGroup,
	ComboboxHeader,
	ComboboxHiddenInput,
	ComboboxOption,
	ComboboxOptions,
	ComboboxSearch,
	ComboboxTarget,
)
from .core.combobox.multi_select import (
	MultiSelect,
)
from .core.combobox.pill import (
	Pill,
	PillGroup,
)
from .core.combobox.pills_input import (
	PillsInput,
	PillsInputField,
)
from .core.combobox.select import (
	Select,
)
from .core.combobox.tags_input import (
	TagsInput,
)
from .core.data_display.accordion import (
	Accordion,
	AccordionChevron,
	AccordionControl,
	AccordionItem,
	AccordionPanel,
)
from .core.data_display.avatar import (
	Avatar,
	AvatarGroup,
)
from .core.data_display.background_image import (
	BackgroundImage,
)
from .core.data_display.badge import (
	Badge,
)
from .core.data_display.card import (
	Card,
	CardSection,
)
from .core.data_display.color_swatch import (
	ColorSwatch,
)
from .core.data_display.image import (
	Image,
)
from .core.data_display.indicator import (
	Indicator,
)
from .core.data_display.kbd import (
	Kbd,
)
from .core.data_display.number_formatter import (
	NumberFormatter,
)
from .core.data_display.spoiler import (
	Spoiler,
)
from .core.data_display.theme_icon import (
	ThemeIcon,
)
from .core.data_display.timeline import (
	Timeline,
	TimelineItem,
)
from .core.feedback.alert import (
	Alert,
)
from .core.feedback.loader import (
	Loader,
)
from .core.feedback.notification import (
	Notification,
)
from .core.feedback.progress import (
	Progress,
	ProgressLabel,
	ProgressRoot,
	ProgressSection,
)
from .core.feedback.ring_progress import (
	RingProgress,
)
from .core.feedback.semi_circle_progress import (
	SemiCircleProgress,
)
from .core.feedback.skeleton import (
	Skeleton,
)
from .core.inputs.angle_slider import (
	AngleSlider,
)
from .core.inputs.checkbox import (
	Checkbox,
	CheckboxCard,
	CheckboxGroup,
	CheckboxIndicator,
)
from .core.inputs.chip import (
	Chip,
	ChipGroup,
)
from .core.inputs.color_input import (
	ColorInput,
)
from .core.inputs.color_picker import (
	ColorPicker,
)
from .core.inputs.fieldset import (
	Fieldset,
)
from .core.inputs.file_input import (
	FileInput,
)
from .core.inputs.input import (
	Input,
	InputClearButton,
	InputDescription,
	InputError,
	InputLabel,
	InputPlaceholder,
	InputWrapper,
)
from .core.inputs.json_input import (
	JsonInput,
)
from .core.inputs.native_select import (
	NativeSelect,
)
from .core.inputs.number_input import (
	NumberInput,
)
from .core.inputs.password_input import (
	PasswordInput,
)
from .core.inputs.pin_input import (
	PinInput,
)
from .core.inputs.radio import (
	Radio,
	RadioCard,
	RadioGroup,
	RadioIndicator,
)
from .core.inputs.range_slider import (
	RangeSlider,
)
from .core.inputs.rating import (
	Rating,
)
from .core.inputs.segmented_control import (
	SegmentedControl,
)
from .core.inputs.slider import (
	Slider,
)
from .core.inputs.switch import (
	Switch,
)
from .core.inputs.text_input import (
	TextInput,
)
from .core.inputs.textarea import (
	Textarea,
)
from .core.layout.appshell import (
	AppShell,
	AppShellAside,
	AppShellAsideAttributes,
	AppShellAsideClassNames,
	AppShellAsideConfiguration,
	AppShellAsideProps,
	AppShellAsideStyles,
	AppShellAsideStylesNames,
	AppShellAttributes,
	AppShellClassNames,
	AppShellCollapsed,
	AppShellCompoundProps,
	AppShellCSSVariables,
	AppShellFooter,
	AppShellFooterAttributes,
	AppShellFooterClassNames,
	AppShellFooterConfiguration,
	AppShellFooterProps,
	AppShellFooterStyles,
	AppShellFooterStylesNames,
	AppShellHeader,
	AppShellHeaderAttributes,
	AppShellHeaderClassNames,
	AppShellHeaderConfiguration,
	AppShellHeaderProps,
	AppShellHeaderStyles,
	AppShellHeaderStylesNames,
	AppShellLayout,
	AppShellMain,
	AppShellMainClassNames,
	AppShellMainProps,
	AppShellMainStyles,
	AppShellMainStylesNames,
	AppShellNavbar,
	AppShellNavbarAttributes,
	AppShellNavbarClassNames,
	AppShellNavbarConfiguration,
	AppShellNavbarProps,
	AppShellNavbarStyles,
	AppShellNavbarStylesNames,
	AppShellProps,
	AppShellResponsiveSize,
	AppShellRootCSSVariables,
	AppShellSection,
	AppShellSectionClassNames,
	AppShellSectionProps,
	AppShellSectionStyles,
	AppShellSectionStylesNames,
	AppShellSize,
	AppShellStyles,
	AppShellStylesNames,
	TransitionTimingFunction,
)
from .core.layout.aspect_ratio import (
	AspectRatio,
	AspectRatioAttributes,
	AspectRatioClassNames,
	AspectRatioCSSVariables,
	AspectRatioProps,
	AspectRatioStyles,
	AspectRatioStylesNames,
)
from .core.layout.center import (
	Center,
	CenterAttributes,
	CenterClassNames,
	CenterProps,
	CenterStyles,
	CenterStylesNames,
)
from .core.layout.container import (
	Container,
	ContainerAttributes,
	ContainerClassNames,
	ContainerCSSVariables,
	ContainerProps,
	ContainerStyles,
	ContainerStylesNames,
)
from .core.layout.flex import (
	Flex,
	FlexAttributes,
	FlexClassNames,
	FlexProps,
	FlexStyles,
	FlexStylesNames,
)
from .core.layout.grid import (
	ColSpan,
	Grid,
	GridAttributes,
	GridBreakpoints,
	GridClassNames,
	GridCol,
	GridColAttributes,
	GridColClassNames,
	GridColCSSVariables,
	GridColProps,
	GridColStyles,
	GridColStylesNames,
	GridCSSVariables,
	GridProps,
	GridStyles,
	GridStylesNames,
)
from .core.layout.group import (
	Group,
	GroupAttributes,
	GroupClassNames,
	GroupCSSVariables,
	GroupCtx,
	GroupProps,
	GroupStyles,
	GroupStylesNames,
)
from .core.layout.simple_grid import (
	SimpleGrid,
	SimpleGridAttributes,
	SimpleGridClassNames,
	SimpleGridProps,
	SimpleGridStyles,
	SimpleGridStylesNames,
)
from .core.layout.space import (
	Space,
	SpaceProps,
)
from .core.layout.stack import (
	Stack,
	StackAttributes,
	StackClassNames,
	StackCSSVariables,
	StackProps,
	StackStyles,
	StackStylesNames,
)
from .core.misc.collapse import (
	Collapse,
)
from .core.misc.divider import (
	Divider,
)
from .core.misc.focus_trap import (
	FocusTrap,
	FocusTrapInitialFocus,
)
from .core.misc.paper import (
	Paper,
)
from .core.misc.portal import (
	Portal,
)
from .core.misc.scroll_area import (
	ScrollArea,
	ScrollAreaAutosize,
)
from .core.misc.transition import (
	Transition,
)
from .core.misc.visually_hidden import (
	VisuallyHidden,
)
from .core.navigation.anchor import (
	Anchor,
)
from .core.navigation.breadcrumbs import (
	Breadcrumbs,
)
from .core.navigation.burger import (
	Burger,
)
from .core.navigation.nav_link import (
	NavLink,
)
from .core.navigation.pagination import (
	Pagination,
	PaginationControl,
	PaginationDots,
	PaginationFirst,
	PaginationItems,
	PaginationLast,
	PaginationNext,
	PaginationPrevious,
	PaginationRoot,
)
from .core.navigation.stepper import (
	Stepper,
	StepperCompleted,
	StepperStep,
)
from .core.navigation.table_of_contents import (
	TableOfContents,
)
from .core.navigation.tabs import (
	Tabs,
	TabsList,
	TabsPanel,
	TabsTab,
)
from .core.navigation.tree import Tree, TreeState
from .core.overlays.affix import (
	Affix,
)
from .core.overlays.dialog import (
	Dialog,
)
from .core.overlays.drawer import (
	Drawer,
	DrawerBody,
	DrawerCloseButton,
	DrawerContent,
	DrawerHeader,
	DrawerOverlay,
	DrawerRoot,
	DrawerStack,
	DrawerTitle,
)
from .core.overlays.floating_indicator import (
	FloatingIndicator,
)
from .core.overlays.hover_card import (
	HoverCard,
	HoverCardDropdown,
	HoverCardGroup,
	HoverCardTarget,
)
from .core.overlays.loading_overlay import (
	LoadingOverlay,
)
from .core.overlays.menu import (
	Menu,
	MenuDivider,
	MenuDropdown,
	MenuItem,
	MenuLabel,
	MenuSub,
	MenuTarget,
)
from .core.overlays.modal import (
	Modal,
	ModalBody,
	ModalCloseButton,
	ModalContent,
	ModalHeader,
	ModalOverlay,
	ModalRoot,
	ModalStack,
	ModalTitle,
)
from .core.overlays.overlay import (
	Overlay,
)
from .core.overlays.popover import (
	Popover,
	PopoverDropdown,
	PopoverTarget,
)
from .core.overlays.tooltip import (
	Tooltip,
	TooltipFloating,
	TooltipGroup,
)
from .core.provider import (
	ConvertCSSVariablesInput,
	CSSVariablesResolver,
	HeadlessMantineProvider,
	HeadlessMantineProviderProps,
	MantineColorSchemeManager,
	MantineProvider,
	MantineProviderEnvironment,
	MantineProviderForceColorScheme,
	MantineProviderProps,
	MantineStylesTransform,
)
from .core.styles import (
	CSSVariables,
	CSSVariablesDict,
	MantineStyle,
	MantineStyleProp,
	MantineStyleProps,
	StyleFn,
	StyleProp,
)
from .core.theme import (
	HeadingStyle,
	MantineColorsTuple,
	MantineGradient,
	MantineHeadings,
	MantineLineHeight,
	MantineLineHeightValues,
	MantinePrimaryShade,
	MantineRadius,
	MantineRadiusValues,
	MantineShadow,
	MantineShadowsValues,
	MantineSpacing,
	MantineSpacingValues,
	MantineStylesRecord,
	MantineTheme,
	MantineThemeColors,
	MantineThemeComponent,
	MantineThemeComponents,
	MantineThemeOther,
	MantineThemeOverride,
	MantineThemeSizesOverride,
	VariantColorResolverResult,
	VariantColorsResolver,
	VariantColorsResolverInput,
)
from .core.types import (
	DefaultMantineColor,
	DefaultMantineSize,
	MantineBreakpoint,
	MantineBreakpointsValues,
	MantineColor,
	MantineColorScheme,
	MantineColorShade,
	MantineFontFamily,
	MantineFontSize,
	MantineFontSizesValues,
	MantineFontStyle,
	MantinePosition,
	MantineSize,
	MantineTextAlign,
	MantineTextDecoration,
	MantineTextTransform,
	MantineTextWrap,
)
from .core.typography.blockquote import (
	Blockquote,
)
from .core.typography.code import (
	Code,
)
from .core.typography.highlight import (
	Highlight,
)
from .core.typography.list import (
	List,
	ListItem,
)
from .core.typography.mark import (
	Mark,
)
from .core.typography.table import (
	Table,
	TableCaption,
	TableDataRenderer,
	TableScrollContainer,
	TableTbody,
	TableTd,
	TableTfoot,
	TableTh,
	TableThead,
	TableTr,
)
from .core.typography.text import (
	Text,
)
from .core.typography.title import (
	Title,
)
from .core.typography.typography import (
	Typography,
)
from .dates.calendar import (
	Calendar,
)
from .dates.calendar_header import (
	CalendarHeader,
)
from .dates.date_input import (
	DateInput,
)
from .dates.date_picker import (
	DatePicker,
)
from .dates.date_picker_input import (
	DatePickerInput,
)
from .dates.date_time_picker import (
	DateTimePicker,
)
from .dates.dates_provider import (
	DatesProvider,
)
from .dates.day import (
	Day,
)
from .dates.decade_level import (
	DecadeLevel,
)
from .dates.decade_level_group import (
	DecadeLevelGroup,
)
from .dates.hidden_dates_input import (
	HiddenDatesInput,
)
from .dates.levels_group import (
	LevelsGroup,
)
from .dates.mini_calendar import (
	MiniCalendar,
)
from .dates.month import (
	Month,
)
from .dates.month_level import (
	MonthLevel,
)
from .dates.month_level_group import (
	MonthLevelGroup,
)
from .dates.month_picker import (
	MonthPicker,
)
from .dates.month_picker_input import (
	MonthPickerInput,
)
from .dates.months_list import (
	MonthsList,
)
from .dates.picker_control import (
	PickerControl,
)
from .dates.picker_input_base import (
	PickerInputBase,
)
from .dates.time_grid import (
	TimeGrid,
)
from .dates.time_input import (
	TimeInput,
)
from .dates.time_picker import (
	TimePicker,
)
from .dates.time_value import (
	TimeValue,
)
from .dates.weekdays_row import (
	WeekdaysRow,
)
from .dates.year_level import (
	YearLevel,
)
from .dates.year_level_group import (
	YearLevelGroup,
)
from .dates.year_picker import (
	YearPicker,
)
from .dates.year_picker_input import (
	YearPickerInput,
)
from .dates.years_list import (
	YearsList,
)
from .form.form import (
	FieldValue,
	FormValues,
	MantineForm,
	MantineFormProps,
)
from .form.internal import (
	FormInternal,
	FormInternalProps,
	FormMode,
)
from .form.validators import (
	AllowedFileTypes,
	EndsWith,
	HasLength,
	IsAfter,
	IsArrayNotEmpty,
	IsBefore,
	IsDate,
	IsEmail,
	IsInRange,
	IsInteger,
	IsISODate,
	IsJSONString,
	IsNotEmpty,
	IsNotEmptyHTML,
	IsNumber,
	IsULID,
	IsUrl,
	IsUUID,
	Matches,
	MatchesField,
	MaxFileSize,
	MaxItems,
	MinItems,
	RequiredUnless,
	RequiredWhen,
	SerializedValidation,
	SerializedValidationNode,
	ServerValidation,
	StartsWith,
	Validation,
	ValidationNode,
	Validator,
)

__all__ = (
	"Accordion",
	"AccordionChevron",
	"AccordionControl",
	"AccordionItem",
	"AccordionPanel",
	"ActionIcon",
	"ActionIconGroup",
	"ActionIconGroupSection",
	"Affix",
	"Alert",
	"AllowedFileTypes",
	"Anchor",
	"AngleSlider",
	"AppShell",
	"AppShellAside",
	"AppShellAsideAttributes",
	"AppShellAsideClassNames",
	"AppShellAsideConfiguration",
	"AppShellAsideProps",
	"AppShellAsideStyles",
	"AppShellAsideStylesNames",
	"AppShellAttributes",
	"AppShellCSSVariables",
	"AppShellClassNames",
	"AppShellCollapsed",
	"AppShellCompoundProps",
	"AppShellFooter",
	"AppShellFooterAttributes",
	"AppShellFooterClassNames",
	"AppShellFooterConfiguration",
	"AppShellFooterProps",
	"AppShellFooterStyles",
	"AppShellFooterStylesNames",
	"AppShellHeader",
	"AppShellHeaderAttributes",
	"AppShellHeaderClassNames",
	"AppShellHeaderConfiguration",
	"AppShellHeaderProps",
	"AppShellHeaderStyles",
	"AppShellHeaderStylesNames",
	"AppShellLayout",
	"AppShellMain",
	"AppShellMainClassNames",
	"AppShellMainProps",
	"AppShellMainStyles",
	"AppShellMainStylesNames",
	"AppShellNavbar",
	"AppShellNavbarAttributes",
	"AppShellNavbarClassNames",
	"AppShellNavbarConfiguration",
	"AppShellNavbarProps",
	"AppShellNavbarStyles",
	"AppShellNavbarStylesNames",
	"AppShellProps",
	"AppShellResponsiveSize",
	"AppShellRootCSSVariables",
	"AppShellSection",
	"AppShellSectionClassNames",
	"AppShellSectionProps",
	"AppShellSectionStyles",
	"AppShellSectionStylesNames",
	"AppShellSize",
	"AppShellStyles",
	"AppShellStylesNames",
	"AreaChart",
	"AspectRatio",
	"AspectRatioAttributes",
	"AspectRatioCSSVariables",
	"AspectRatioClassNames",
	"AspectRatioProps",
	"AspectRatioStyles",
	"AspectRatioStylesNames",
	"Autocomplete",
	"Avatar",
	"AvatarGroup",
	"BackgroundImage",
	"Badge",
	"BarChart",
	"Blockquote",
	"Box",
	"BoxComponentProps",
	"BoxMod",
	"BoxProps",
	"Breadcrumbs",
	"BubbleChart",
	"Burger",
	"Button",
	"ButtonGroup",
	"ButtonGroupSection",
	"CSSVariables",
	"CSSVariablesDict",
	"CSSVariablesResolver",
	"Calendar",
	"CalendarHeader",
	"Card",
	"CardSection",
	"Center",
	"CenterAttributes",
	"CenterClassNames",
	"CenterProps",
	"CenterStyles",
	"CenterStylesNames",
	"ChartLegend",
	"ChartTooltip",
	"Checkbox",
	"CheckboxCard",
	"CheckboxGroup",
	"CheckboxIndicator",
	"Chip",
	"ChipGroup",
	"CloseButton",
	"Code",
	"ColSpan",
	"Collapse",
	"ColorInput",
	"ColorPicker",
	"ColorSwatch",
	"Combobox",
	"ComboboxChevron",
	"ComboboxClearButton",
	"ComboboxDropdown",
	"ComboboxDropdownTarget",
	"ComboboxEmpty",
	"ComboboxEventsTarget",
	"ComboboxFooter",
	"ComboboxGroup",
	"ComboboxHeader",
	"ComboboxHiddenInput",
	"ComboboxOption",
	"ComboboxOptions",
	"ComboboxSearch",
	"ComboboxTarget",
	"CompositeChart",
	"Container",
	"ContainerAttributes",
	"ContainerCSSVariables",
	"ContainerClassNames",
	"ContainerProps",
	"ContainerStyles",
	"ContainerStylesNames",
	"ConvertCSSVariablesInput",
	"CopyButton",
	"DateInput",
	"DatePicker",
	"DatePickerInput",
	"DateTimePicker",
	"DatesProvider",
	"Day",
	"DecadeLevel",
	"DecadeLevelGroup",
	"DefaultMantineColor",
	"DefaultMantineSize",
	"Dialog",
	"Divider",
	"DonutChart",
	"Drawer",
	"DrawerBody",
	"DrawerCloseButton",
	"DrawerContent",
	"DrawerHeader",
	"DrawerOverlay",
	"DrawerRoot",
	"DrawerStack",
	"DrawerTitle",
	"EndsWith",
	"FieldValue",
	"Fieldset",
	"FileButton",
	"FileInput",
	"Flex",
	"FlexAttributes",
	"FlexClassNames",
	"FlexProps",
	"FlexStyles",
	"FlexStylesNames",
	"FloatingIndicator",
	"FocusTrap",
	"FocusTrapInitialFocus",
	"FormInternal",
	"FormInternalProps",
	"FormMode",
	"FormValues",
	"FunnelChart",
	"Grid",
	"GridAttributes",
	"GridBreakpoints",
	"GridCSSVariables",
	"GridClassNames",
	"GridCol",
	"GridColAttributes",
	"GridColCSSVariables",
	"GridColClassNames",
	"GridColProps",
	"GridColStyles",
	"GridColStylesNames",
	"GridProps",
	"GridStyles",
	"GridStylesNames",
	"Group",
	"GroupAttributes",
	"GroupCSSVariables",
	"GroupClassNames",
	"GroupCtx",
	"GroupProps",
	"GroupStyles",
	"GroupStylesNames",
	"HasLength",
	"HeadingStyle",
	"HeadlessMantineProvider",
	"HeadlessMantineProviderProps",
	"Heatmap",
	"HiddenDatesInput",
	"Highlight",
	"HoverCard",
	"HoverCardDropdown",
	"HoverCardGroup",
	"HoverCardTarget",
	"Image",
	"Indicator",
	"Input",
	"InputClearButton",
	"InputDescription",
	"InputError",
	"InputLabel",
	"InputPlaceholder",
	"InputWrapper",
	"IsAfter",
	"IsArrayNotEmpty",
	"IsBefore",
	"IsDate",
	"IsEmail",
	"IsISODate",
	"IsInRange",
	"IsInteger",
	"IsJSONString",
	"IsNotEmpty",
	"IsNotEmptyHTML",
	"IsNumber",
	"IsULID",
	"IsUUID",
	"IsUrl",
	"JsonInput",
	"Kbd",
	"LevelsGroup",
	"LineChart",
	"List",
	"ListItem",
	"Loader",
	"LoadingOverlay",
	"MantineBreakpoint",
	"MantineBreakpointsValues",
	"MantineColor",
	"MantineColorScheme",
	"MantineColorSchemeManager",
	"MantineColorShade",
	"MantineColorsTuple",
	"MantineComponentProps",
	"MantineFontFamily",
	"MantineFontSize",
	"MantineFontSizesValues",
	"MantineFontStyle",
	"MantineForm",
	"MantineFormProps",
	"MantineGradient",
	"MantineHeadings",
	"MantineLineHeight",
	"MantineLineHeightValues",
	"MantinePosition",
	"MantinePrimaryShade",
	"MantineProvider",
	"MantineProviderEnvironment",
	"MantineProviderForceColorScheme",
	"MantineProviderProps",
	"MantineRadius",
	"MantineRadiusValues",
	"MantineShadow",
	"MantineShadowsValues",
	"MantineSize",
	"MantineSpacing",
	"MantineSpacingValues",
	"MantineStyle",
	"MantineStyleProp",
	"MantineStyleProps",
	"MantineStylesRecord",
	"MantineStylesTransform",
	"MantineTextAlign",
	"MantineTextDecoration",
	"MantineTextTransform",
	"MantineTextWrap",
	"MantineTheme",
	"MantineThemeColors",
	"MantineThemeComponent",
	"MantineThemeComponents",
	"MantineThemeOther",
	"MantineThemeOverride",
	"MantineThemeSizesOverride",
	"Mark",
	"Matches",
	"MatchesField",
	"MaxFileSize",
	"MaxItems",
	"Menu",
	"MenuDivider",
	"MenuDropdown",
	"MenuItem",
	"MenuLabel",
	"MenuSub",
	"MenuTarget",
	"MinItems",
	"MiniCalendar",
	"Mod",
	"Modal",
	"ModalBody",
	"ModalCloseButton",
	"ModalContent",
	"ModalHeader",
	"ModalOverlay",
	"ModalRoot",
	"ModalStack",
	"ModalTitle",
	"Month",
	"MonthLevel",
	"MonthLevelGroup",
	"MonthPicker",
	"MonthPickerInput",
	"MonthsList",
	"MultiSelect",
	"NativeSelect",
	"NavLink",
	"Notification",
	"NumberFormatter",
	"NumberInput",
	"Overlay",
	"Pagination",
	"PaginationControl",
	"PaginationDots",
	"PaginationFirst",
	"PaginationItems",
	"PaginationLast",
	"PaginationNext",
	"PaginationPrevious",
	"PaginationRoot",
	"Paper",
	"PasswordInput",
	"PickerControl",
	"PickerInputBase",
	"PieChart",
	"Pill",
	"PillGroup",
	"PillsInput",
	"PillsInputField",
	"PinInput",
	"Popover",
	"PopoverDropdown",
	"PopoverTarget",
	"Portal",
	"Progress",
	"ProgressLabel",
	"ProgressRoot",
	"ProgressSection",
	"RadarChart",
	"RadialBarChart",
	"Radio",
	"RadioCard",
	"RadioGroup",
	"RadioIndicator",
	"RangeSlider",
	"Rating",
	"RequiredUnless",
	"RequiredWhen",
	"RingProgress",
	"ScatterChart",
	"ScrollArea",
	"ScrollAreaAutosize",
	"SegmentedControl",
	"Select",
	"SemiCircleProgress",
	"SerializedValidation",
	"SerializedValidationNode",
	"ServerValidation",
	"SimpleGrid",
	"SimpleGridAttributes",
	"SimpleGridClassNames",
	"SimpleGridProps",
	"SimpleGridStyles",
	"SimpleGridStylesNames",
	"Skeleton",
	"Slider",
	"Space",
	"SpaceProps",
	"Sparkline",
	"Spoiler",
	"Stack",
	"StackAttributes",
	"StackCSSVariables",
	"StackClassNames",
	"StackProps",
	"StackStyles",
	"StackStylesNames",
	"StartsWith",
	"Stepper",
	"StepperCompleted",
	"StepperStep",
	"StyleFn",
	"StyleProp",
	"Switch",
	"Table",
	"TableCaption",
	"TableDataRenderer",
	"TableOfContents",
	"TableScrollContainer",
	"TableTbody",
	"TableTd",
	"TableTfoot",
	"TableTh",
	"TableThead",
	"TableTr",
	"Tabs",
	"TabsList",
	"TabsPanel",
	"TabsTab",
	"TagsInput",
	"Text",
	"TextInput",
	"Textarea",
	"ThemeIcon",
	"TimeGrid",
	"TimeInput",
	"TimePicker",
	"TimeValue",
	"Timeline",
	"TimelineItem",
	"Title",
	"Tooltip",
	"TooltipFloating",
	"TooltipGroup",
	"Transition",
	"TransitionTimingFunction",
	"Tree",
	"TreeState",
	"Typography",
	"UnstyledButton",
	"Validation",
	"ValidationNode",
	"Validator",
	"VariantColorResolverResult",
	"VariantColorsResolver",
	"VariantColorsResolverInput",
	"VisuallyHidden",
	"WeekdaysRow",
	"YearLevel",
	"YearLevelGroup",
	"YearPicker",
	"YearPickerInput",
	"YearsList",
)



from typing import Any

import pulse as ps


@ps.react_component("List", "@mantine/core")
def List(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("List", "@mantine/core", prop="Item")
def ListItem(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Pagination", "@mantine/core")
def Pagination(key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Root")
def PaginationRoot(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Control")
def PaginationControl(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Dots")
def PaginationDots(key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="First")
def PaginationFirst(key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Last")
def PaginationLast(key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Next")
def PaginationNext(key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Previous")
def PaginationPrevious(key: str | None = None, **props: Any): ...


@ps.react_component("Pagination", "@mantine/core", prop="Items")
def PaginationItems(*children: ps.Child, key: str | None = None, **props: Any): ...



from collections.abc import Sequence
from typing import Any, Unpack

import pulse as ps

from .base import MantineComponentProps
from .styles import CSSVariables, MantineStyleProp, MantineStyleProps
from .types import MantineBreakpoint

Mod = dict[str, Any] | str
BoxMod = Mod | Sequence[Mod] | Sequence["BoxMod"]


class BoxProps(MantineStyleProps, total=False):
	className: ps.ClassName  # str or CssReference
	"Class added to the root element, if applicable"
	style: MantineStyleProp
	"Inline style added to root component element, can subscribe to theme defined on MantineProvider"
	__vars: CSSVariables
	"CSS variables defined on root component element"
	__size: str
	"`size` property passed down the HTML element"
	hiddenFrom: MantineBreakpoint
	"Breakpoint above which the component is hidden with `display: none`"
	visibleFrom: MantineBreakpoint
	"Breakpoint below which the component is hidden with `display: none`"
	lightHidden: bool
	"Determines whether component should be hidden in light color scheme with `display: none`"
	darkHidden: bool
	"Determines whether component should be hidden in dark color scheme with `display: none`"
	mod: BoxMod
	"Element modifiers transformed into `data-` attributes, for example, `{ 'data-size': 'xl' }`, falsy values are removed"


class BoxComponentProps(BoxProps, MantineComponentProps, total=False):
	variant: str
	"Variant passed from parent component, sets `data-variant`"
	size: str | float
	"Size passed from parent component, sets `data-size` if value is not number like"


@ps.react_component("Box", "@mantine/core")
def Box(
	*children: ps.Child, key: str | None = None, **props: Unpack[BoxComponentProps]
): ...



from typing import Any

import pulse as ps


@ps.react_component("ActionIcon", "@mantine/core")
def ActionIcon(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("ActionIcon", "@mantine/core", prop="Group")
def ActionIconGroup(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("ActionIcon", "@mantine/core", prop="GroupSection")
def ActionIconGroupSection(
	*children: ps.Child, key: str | None = None, **props: Any
): ...



from typing import Any

import pulse as ps


@ps.react_component("Table", "@mantine/core")
def Table(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Thead")
def TableThead(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Tbody")
def TableTbody(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Tfoot")
def TableTfoot(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Td")
def TableTd(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Th")
def TableTh(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Tr")
def TableTr(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="Caption")
def TableCaption(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="ScrollContainer")
def TableScrollContainer(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Table", "@mantine/core", prop="DataRenderer")
def TableDataRenderer(key: str | None = None, **props: Any): ...



from __future__ import annotations

from typing import Any, Literal, TypedDict, Unpack

import pulse as ps

from ..box import BoxProps
from ..styles import StyleFn, StyleProp
from ..theme import MantineSpacing

StackStylesNames = Literal["root"]
StackAttributes = dict[StackStylesNames, dict[str, Any]]
StackStyles = dict[StackStylesNames, ps.CSSProperties]
StackClassNames = dict[StackStylesNames, ps.ClassName]


class StackCSSVariables(TypedDict, total=False):
	root: dict[Literal["--stack-gap", "--stack-align", "--stack-justify"], str]


class StackProps(ps.HTMLDivProps, BoxProps, total=False):  # pyright: ignore[reportIncompatibleVariableOverride]
	gap: StyleProp[MantineSpacing]
	"""Key of `theme.spacing` or any valid CSS value to set `gap` property, numbers are converted to rem @default `'md'`"""
	align: str
	"""Controls `align-items` CSS property @default `'stretch'`"""
	justify: str
	"""Controls `justify-content` CSS property @default `'flex-start'`"""

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	variant: str
	"""Component variant, if applicable"""
	classNames: StackClassNames | StyleFn[StackProps, Any, StackClassNames]
	"""Additional class names passed to elements"""
	styles: StackStyles | StyleFn[StackProps, Any, StackStyles]
	"""Additional styles passed to elements"""
	vars: StackCSSVariables
	"""CSS variables for the component"""
	attributes: StackAttributes
	"""Additional attributes passed to elements"""


@ps.react_component("Stack", "@mantine/core")
def Stack(*children: ps.Child, key: str | None = None, **props: Unpack[StackProps]): ...



from typing import Any

import pulse as ps


@ps.react_component("Drawer", "@mantine/core")
def Drawer(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Root")
def DrawerRoot(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Overlay")
def DrawerOverlay(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Content")
def DrawerContent(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Body")
def DrawerBody(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Header")
def DrawerHeader(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Title")
def DrawerTitle(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="CloseButton")
def DrawerCloseButton(key: str | None = None, **props: Any): ...


@ps.react_component("Drawer", "@mantine/core", prop="Stack")
def DrawerStack(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Progress", "@mantine/core")
def Progress(key: str | None = None, **props: Any): ...


@ps.react_component("Progress", "@mantine/core", prop="Section")
def ProgressSection(key: str | None = None, **props: Any): ...


@ps.react_component("Progress", "@mantine/core", prop="Root")
def ProgressRoot(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Progress", "@mantine/core", prop="Label")
def ProgressLabel(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Popover", "@mantine/core")
def Popover(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Popover", "@mantine/core", prop="Target")
def PopoverTarget(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Popover", "@mantine/core", prop="Dropdown")
def PopoverDropdown(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Radio", "@mantine/core")
def Radio(key: str | None = None, **props: Any): ...


# Only Radio component that needs to be registered as a form input
@ps.react_component("RadioGroup", "pulse-mantine")
def RadioGroup(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Radio", "@mantine/core", prop="Card")
def RadioCard(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Radio", "@mantine/core", prop="Indicator")
def RadioIndicator(*children: ps.Child, key: str | None = None, **props: Any): ...



from __future__ import annotations

from typing import Any, Literal, Unpack

import pulse as ps

from ..box import BoxProps
from ..styles import StyleFn, StyleProp
from ..theme import MantineSpacing

SimpleGridStylesNames = Literal["root", "container"]
SimpleGridAttributes = dict[SimpleGridStylesNames, dict[str, Any]]
SimpleGridStyles = dict[SimpleGridStylesNames, ps.CSSProperties]
SimpleGridClassNames = dict[SimpleGridStylesNames, str]


class SimpleGridProps(ps.HTMLDivProps, BoxProps, total=False):  # pyright: ignore[reportIncompatibleVariableOverride]
	cols: StyleProp[int]
	"""Number of columns @default `1`"""
	spacing: StyleProp[MantineSpacing]
	"""Spacing between columns @default `'md'`"""
	verticalSpacing: StyleProp[MantineSpacing]
	"""Spacing between rows @default `'md'`"""
	type: Literal["media", "container"]
	"""Determines typeof of queries that are used for responsive styles @default `'media'`"""

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	variant: str
	"""Component variant, if applicable"""
	classNames: (
		SimpleGridClassNames | StyleFn[SimpleGridProps, Any, SimpleGridClassNames]
	)
	"""Additional class names passed to elements"""
	styles: SimpleGridStyles | StyleFn[SimpleGridProps, Any, SimpleGridStyles]
	"""Additional styles passed to elements"""
	# no vars
	attributes: SimpleGridAttributes
	"""Additional attributes passed to elements"""


@ps.react_component("SimpleGrid", "@mantine/core")
def SimpleGrid(
	*children: ps.Child, key: str | None = None, **props: Unpack[SimpleGridProps]
): ...



from typing import Any

import pulse as ps


@ps.react_component("Card", "@mantine/core")
def Card(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Card", "@mantine/core", prop="Section")
def CardSection(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Stepper", "@mantine/core")
def Stepper(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Stepper", "@mantine/core", prop="Step")
def StepperStep(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Stepper", "@mantine/core", prop="Completed")
def StepperCompleted(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Accordion", "@mantine/core")
def Accordion(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Accordion", "@mantine/core", prop="Item")
def AccordionItem(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Accordion", "@mantine/core", prop="Panel")
def AccordionPanel(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Accordion", "@mantine/core", prop="Control")
def AccordionControl(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Accordion", "@mantine/core", prop="Chevron")
def AccordionChevron(key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Button", "@mantine/core")
def Button(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Button", "@mantine/core", prop="Group")
def ButtonGroup(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Button", "@mantine/core", prop="GroupSection")
def ButtonGroupSection(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Checkbox", "pulse-mantine")
def Checkbox(key: str | None = None, **props: Any): ...


@ps.react_component("Checkbox", "@mantine/core", prop="Group")
def CheckboxGroup(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Checkbox", "@mantine/core", prop="Indicator")
def CheckboxIndicator(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Checkbox", "@mantine/core", prop="Card")
def CheckboxCard(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Menu", "@mantine/core")
def Menu(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Menu", "@mantine/core", prop="Item")
def MenuItem(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Menu", "@mantine/core", prop="Label")
def MenuLabel(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Menu", "@mantine/core", prop="Dropdown")
def MenuDropdown(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Menu", "@mantine/core", prop="Target")
def MenuTarget(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Menu", "@mantine/core", prop="Divider")
def MenuDivider(key: str | None = None, **props: Any): ...


@ps.react_component("Menu", "@mantine/core", prop="Sub")
def MenuSub(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps
from pulse.codegen.imports import ImportStatement


@ps.react_component(
	"DatesProvider",
	"pulse-mantine",
	extra_imports=[ImportStatement(src="@mantine/dates/styles.css", side_effect=True)],
)
def DatesProvider(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Avatar", "@mantine/core")
def Avatar(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Avatar", "@mantine/core", prop="Group")
def AvatarGroup(*children: ps.Child, key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps


@ps.react_component("Chip", "pulse-mantine")
def Chip(key: str | None = None, **props: Any): ...


@ps.react_component("Chip", "@mantine/core", prop="Group")
def ChipGroup(*children: ps.Child, key: str | None = None, **props: Any): ...



from __future__ import annotations

from typing import Any, Literal, Unpack

import pulse as ps

from ..box import BoxProps
from ..styles import StyleFn

CenterStylesNames = Literal["root"]
CenterAttributes = dict[CenterStylesNames, dict[str, Any]]
CenterStyles = dict[CenterStylesNames, ps.CSSProperties]
CenterClassNames = dict[CenterStylesNames, str]


class CenterProps(ps.HTMLDivProps, BoxProps, total=False):  # pyright: ignore[reportIncompatibleVariableOverride]
	inline: bool
	"If set, `inline-flex` is used instead of `flex`"

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	variant: str
	"""Component variant, if applicable"""
	classNames: CenterClassNames | StyleFn[CenterProps, Any, CenterClassNames]
	"""Additional class names passed to elements"""
	styles: CenterStyles | StyleFn[CenterProps, Any, CenterStyles]
	"""Additional styles passed to elements"""
	# no vars
	attributes: CenterAttributes
	"""Additional attributes passed to elements"""


@ps.react_component("Center", "@mantine/core")
def Center(
	*children: ps.Child, key: str | None = None, **props: Unpack[CenterProps]
): ...



from typing import Any

import pulse as ps


@ps.react_component("Modal", "@mantine/core")
def Modal(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Root")
def ModalRoot(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Overlay")
def ModalOverlay(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Content")
def ModalContent(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Body")
def ModalBody(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Header")
def ModalHeader(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Title")
def ModalTitle(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="CloseButton")
def ModalCloseButton(key: str | None = None, **props: Any): ...


@ps.react_component("Modal", "@mantine/core", prop="Stack")
def ModalStack(*children: ps.Child, key: str | None = None, **props: Any): ...



from __future__ import annotations

from typing import Any, Literal, NotRequired, TypedDict

import pulse as ps

from .types import (
	MantineBreakpointsValues,
	MantineColor,
	MantineColorShade,
	MantineFontSizesValues,
	MantineTextWrap,
)


class VariantColorsResolverInput(TypedDict):
	color: MantineColor | None
	theme: MantineTheme
	variant: str
	gradient: NotRequired[MantineGradient]
	autoContrast: NotRequired[bool]


class VariantColorResolverResult(TypedDict):
	background: str
	hover: str
	color: str
	border: str
	hoverColor: NotRequired[str]


VariantColorsResolver = ps.JsFunction[
	[VariantColorsResolverInput], VariantColorResolverResult
]


class MantineTheme(TypedDict):
	focusRing: Literal["auto", "always", "never"]
	"""Controls focus ring styles. Supports the following options:
    *  - `auto` – focus ring is displayed only when the user navigates with keyboard (default value)
    *  - `always` – focus ring is displayed when the user navigates with keyboard and mouse
    *  - `never` – focus ring is always hidden (not recommended)
    """
	scale: float
	"""Rem units scale, change if you customize font-size of `<html />` element
     *  default value is `1` (for `100%`/`16px` font-size on `<html />`)
     """
	fontSmoothing: bool
	"""Determines whether `font-smoothing` property should be set on the body, `true` by default"""
	white: str
	"""White color"""
	black: str
	"""Black color"""
	colors: MantineThemeColors
	"""Object of colors, key is color name, value is an array of at least 10 strings (colors)"""
	primaryShade: MantineColorShade | MantinePrimaryShade
	"""Index of theme.colors[color].
     *  Primary shade is used in all components to determine which color from theme.colors[color] should be used.
     *  Can be either a number (0–9) or an object to specify different color shades for light and dark color schemes.
     *  Default value `{ light: 6, dark: 8 }`
     *
     *  For example,
     *  { primaryShade: 6 } // shade 6 is used both for dark and light color schemes
     *  { primaryShade: { light: 6, dark: 7 } } // different shades for dark and light color schemes
     """
	primaryColor: str
	"""Key of `theme.colors`, hex/rgb/hsl values are not supported.
     *  Determines which color will be used in all components by default.
     *  Default value – `blue`.
     """
	variantColorResolver: VariantColorsResolver
	"""Function to resolve colors based on variant.
     *  Can be used to deeply customize how colors are applied to `Button`, `ActionIcon`, `ThemeIcon`
     *  and other components that use colors from theme.
     """
	autoContrast: bool
	"""Determines whether text color must be changed based on the given `color` prop in filled variant
     *  For example, if you pass `color="blue.1"` to Button component, text color will be changed to `var(--mantine-color-black)`
     *  Default value – `false`
     """
	luminanceThreshold: float
	"""Determines which luminance value is used to determine if text color should be light or dark.
     *  Used only if `theme.autoContrast` is set to `true`.
     *  Default value is `0.3`
     """
	fontFamily: str
	"""Font-family used in all components, system fonts by default"""
	fontFamilyMonospace: str
	"""Monospace font-family, used in code and other similar components, system fonts by default"""
	headings: MantineHeadings
	"""Controls various styles of h1-h6 elements, used in Typography and Title components"""
	radius: MantineRadiusValues
	"""Object of values that are used to set `border-radius` in all components that support it"""
	defaultRadius: MantineRadius
	"""Key of `theme.radius` or any valid CSS value. Default `border-radius` used by most components"""
	spacing: MantineSpacingValues
	"""Object of values that are used to set various CSS properties that control spacing between elements"""
	fontSizes: MantineFontSizesValues
	"""Object of values that are used to control `font-size` property in all components"""
	lineHeights: MantineLineHeightValues
	"""Object of values that are used to control `line-height` property in `Text` component"""
	breakpoints: MantineBreakpointsValues
	"""Object of values that are used to control breakpoints in all components,
     *  values are expected to be defined in em
     """
	shadows: MantineShadowsValues
	"""Object of values that are used to add `box-shadow` styles to components that support `shadow` prop"""
	respectReducedMotion: bool
	"""Determines whether user OS settings to reduce motion should be respected, `false` by default"""
	cursorType: Literal["default", "pointer"]
	"""Determines which cursor type will be used for interactive elements
     * - `default` – cursor that is used by native HTML elements, for example, `input[type="checkbox"]` has `cursor: default` styles
     * - `pointer` – sets `cursor: pointer` on interactive elements that do not have these styles by default
     """
	defaultGradient: MantineGradient
	"""Default gradient configuration for components that support `variant="gradient"`"""
	activeClassName: str
	"""Class added to the elements that have active styles, for example, `Button` and `ActionIcon`"""
	focusClassName: str
	"""Class added to the elements that have focus styles, for example, `Button` or `ActionIcon`.
     *  Overrides `theme.focusRing` property.
     """
	components: MantineThemeComponents
	"""Allows adding `classNames`, `styles` and `defaultProps` to any component"""
	other: MantineThemeOther
	"""Any other properties that you want to access with the theme objects"""


class MantineThemeOverride(TypedDict, total=False):
	"""Partial version of MantineTheme."""

	focusRing: Literal["auto", "always", "never"]
	"""Controls focus ring styles. Supports the following options:
    *  - `auto` – focus ring is displayed only when the user navigates with keyboard (default value)
    *  - `always` – focus ring is displayed when the user navigates with keyboard and mouse
    *  - `never` – focus ring is always hidden (not recommended)
    """
	scale: float
	"""Rem units scale, change if you customize font-size of `<html />` element
     *  default value is `1` (for `100%`/`16px` font-size on `<html />`)
     """
	fontSmoothing: bool
	"""Determines whether `font-smoothing` property should be set on the body, `true` by default"""
	white: str
	"""White color"""
	black: str
	"""Black color"""
	colors: MantineThemeColors
	"""Object of colors, key is color name, value is an array of at least 10 strings (colors)"""
	primaryShade: MantineColorShade | MantinePrimaryShade
	"""Index of theme.colors[color].
     *  Primary shade is used in all components to determine which color from theme.colors[color] should be used.
     *  Can be either a number (0–9) or an object to specify different color shades for light and dark color schemes.
     *  Default value `{ light: 6, dark: 8 }`
     *
     *  For example,
     *  { primaryShade: 6 } // shade 6 is used both for dark and light color schemes
     *  { primaryShade: { light: 6, dark: 7 } } // different shades for dark and light color schemes
     """
	primaryColor: str
	"""Key of `theme.colors`, hex/rgb/hsl values are not supported.
     *  Determines which color will be used in all components by default.
     *  Default value – `blue`.
     """
	variantColorResolver: VariantColorsResolver
	"""Function to resolve colors based on variant.
     *  Can be used to deeply customize how colors are applied to `Button`, `ActionIcon`, `ThemeIcon`
     *  and other components that use colors from theme.
     """
	autoContrast: bool
	"""Determines whether text color must be changed based on the given `color` prop in filled variant
     *  For example, if you pass `color="blue.1"` to Button component, text color will be changed to `var(--mantine-color-black)`
     *  Default value – `false`
     """
	luminanceThreshold: float
	"""Determines which luminance value is used to determine if text color should be light or dark.
     *  Used only if `theme.autoContrast` is set to `true`.
     *  Default value is `0.3`
     """
	fontFamily: str
	"""Font-family used in all components, system fonts by default"""
	fontFamilyMonospace: str
	"""Monospace font-family, used in code and other similar components, system fonts by default"""
	headings: MantineHeadings
	"""Controls various styles of h1-h6 elements, used in Typography and Title components"""
	radius: MantineRadiusValues
	"""Object of values that are used to set `border-radius` in all components that support it"""
	defaultRadius: MantineRadius
	"""Key of `theme.radius` or any valid CSS value. Default `border-radius` used by most components"""
	spacing: MantineSpacingValues
	"""Object of values that are used to set various CSS properties that control spacing between elements"""
	fontSizes: MantineFontSizesValues
	"""Object of values that are used to control `font-size` property in all components"""
	lineHeights: MantineLineHeightValues
	"""Object of values that are used to control `line-height` property in `Text` component"""
	breakpoints: MantineBreakpointsValues
	"""Object of values that are used to control breakpoints in all components,
     *  values are expected to be defined in em
     """
	shadows: MantineShadowsValues
	"""Object of values that are used to add `box-shadow` styles to components that support `shadow` prop"""
	respectReducedMotion: bool
	"""Determines whether user OS settings to reduce motion should be respected, `false` by default"""
	cursorType: Literal["default", "pointer"]
	"""Determines which cursor type will be used for interactive elements
     * - `default` – cursor that is used by native HTML elements, for example, `input[type="checkbox"]` has `cursor: default` styles
     * - `pointer` – sets `cursor: pointer` on interactive elements that do not have these styles by default
     """
	defaultGradient: MantineGradient
	"""Default gradient configuration for components that support `variant="gradient"`"""
	activeClassName: str
	"""Class added to the elements that have active styles, for example, `Button` and `ActionIcon`"""
	focusClassName: str
	"""Class added to the elements that have focus styles, for example, `Button` or `ActionIcon`.
     *  Overrides `theme.focusRing` property.
     """
	components: MantineThemeComponents
	"""Allows adding `classNames`, `styles` and `defaultProps` to any component"""
	other: MantineThemeOther
	"""Any other properties that you want to access with the theme objects"""


MantineStylesRecord = dict[str, ps.CSSProperties]


class MantineThemeComponent(TypedDict, total=False):
	classNames: Any
	styles: Any
	vars: Any
	defaultProps: Any


MantineThemeComponents = dict[str, MantineThemeComponent]


class HeadingStyle(TypedDict):
	fontSize: str
	fontWeight: NotRequired[str]
	lineHeight: str


class MantineThemeSizesOverride(TypedDict):
	pass


class MantineHeadings(TypedDict):
	fontFamily: str
	fontWeight: str
	textWrap: MantineTextWrap
	sizes: dict[str, HeadingStyle]


MantineRadius = str | float  # Simplified from complex conditional type
MantineRadiusValues = dict[str, str]  # Simplified from complex conditional type
MantineSpacing = str | float  # Simplified from complex conditional type
MantineSpacingValues = dict[str, str]  # Simplified from complex conditional type
MantineShadow = str  # Simplified from complex conditional type
MantineShadowsValues = dict[str, str]  # Simplified from complex conditional type
MantineLineHeight = str  # Simplified from complex conditional type
MantineLineHeightValues = dict[str, str]  # Simplified from complex conditional type


class MantineThemeOther(TypedDict):
	"""Any other properties that you want to access with the theme objects"""

	pass


class MantineGradient(TypedDict):
	from_: str
	to: str
	deg: NotRequired[float]


MantineColorsTuple = tuple[str, ...]


class MantinePrimaryShade(TypedDict):
	light: MantineColorShade
	dark: MantineColorShade


MantineThemeColors = dict[
	str, MantineColorsTuple
]  # Simplified from complex conditional type



from typing import Any

import pulse as ps


@ps.react_component("Combobox", "@mantine/core")
def Combobox(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Target")
def ComboboxTarget(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Dropdown")
def ComboboxDropdown(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Options")
def ComboboxOptions(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Option")
def ComboboxOption(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Search")
def ComboboxSearch(key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Empty")
def ComboboxEmpty(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Chevron")
def ComboboxChevron(key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Footer")
def ComboboxFooter(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="Header")
def ComboboxHeader(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="EventsTarget")
def ComboboxEventsTarget(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="DropdownTarget")
def ComboboxDropdownTarget(
	*children: ps.Child, key: str | None = None, **props: Any
): ...


@ps.react_component("Combobox", "@mantine/core", prop="Group")
def ComboboxGroup(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="ClearButton")
def ComboboxClearButton(key: str | None = None, **props: Any): ...


@ps.react_component("Combobox", "@mantine/core", prop="HiddenInput")
def ComboboxHiddenInput(key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps
from pulse.codegen.imports import ImportStatement


@ps.react_component(
	"ChartTooltip",
	"@mantine/charts",
	extra_imports=[ImportStatement(src="@mantine/charts/styles.css", side_effect=True)],
)
def ChartTooltip(key: str | None = None, **props: Any): ...



from typing import (
	Any,
	Literal,
	Required,
	Unpack,
)

import pulse as ps

from pulse_mantine.form.validators import SerializedValidation

FormMode = Literal["controlled", "uncontrolled"]


class FormInternalProps(ps.HTMLFormProps, total=False):
	channelId: Required[str]
	initialValues: dict[str, Any]
	initialErrors: dict[str, Any]
	initialDirty: dict[str, bool]
	initialTouched: dict[str, bool]

	mode: FormMode
	validate: "SerializedValidation"
	validateInputOnBlur: bool | list[str]
	validateInputOnChange: bool | list[str]
	clearInputErrorOnChange: bool
	debounceMs: int
	syncMode: Literal["none", "onBlur", "onChange"]
	syncDebounceMs: int
	action: Required[str]  # pyright: ignore[reportGeneralTypeIssues]

	# Server validation
	onSubmit: ps.EventHandler1[ps.FormEvent[ps.HTMLFormElement]]
	onServerValidation: ps.EventHandler3[Any, dict[str, Any], str]
	onReset: ps.EventHandler1[ps.FormEvent[ps.HTMLFormElement]]


@ps.react_component("Form", "pulse-mantine")
def FormInternal(
	*children: ps.Child, key: str | None = None, **props: Unpack[FormInternalProps]
): ...



"""
Pulse Mantine package version indicator.

Exposes `__version__` which matches the distribution version when installed.
Falls back to reading local pyproject.toml during editable/dev usage.
"""

from __future__ import annotations

from importlib.metadata import PackageNotFoundError
from importlib.metadata import version as _pkg_version
from pathlib import Path

__all__ = ["__version__"]


def _read_local_pyproject_version() -> str | None:
	try:
		here = Path(__file__).resolve()
		root = here.parent.parent  # pulse_mantine/version.py -> pulse_mantine/
		# project pyproject.toml is two levels up: packages/pulse-mantine/python/pyproject.toml
		pyproject = root.parent.parent / "pyproject.toml"
		if not pyproject.exists():
			return None
		for line in pyproject.read_text().splitlines():
			line = line.strip()
			if line.startswith("version") and "=" in line:
				_, rhs = line.split("=", 1)
				rhs = rhs.strip().strip("\"'")
				if rhs:
					return rhs
		return None
	except Exception:
		return None


def _resolve_version() -> str:
	try:
		return _pkg_version("pulse-mantine")
	except PackageNotFoundError:
		pass
	local = _read_local_pyproject_version()
	if local:
		return local
	return "0.0.0"


__version__: str = _resolve_version()



from typing import Any, Unpack, cast

import pulse as ps
from pulse.helpers import call_flexible, create_task, maybe_await

ExpandedState = dict[str, bool]


class MantineTreeProps(ps.HTMLDivProps, total=False):
	data: list[dict[str, Any]]
	levelOffset: int
	selectOnClick: bool
	clearSelectionOnOutsideClick: bool
	className: str | ps.CssReference
	classNames: dict[str, str]
	styles: dict[str, Any]
	style: dict[str, Any]


@ps.react_component("Tree", "pulse-mantine")
def TreeInternal(
	*children: ps.Child,
	key: str | None = None,
	channelId: str | None = None,
	initialExpandedState: ExpandedState | None = None,
	autoSync: bool = True,
	# useTree options (forwarded to the JS wrapper)
	initialSelectedState: list[str] | None = None,
	initialCheckedState: list[str] | None = None,
	multiple: bool | None = None,
	**props: Unpack[MantineTreeProps],
): ...


class TreeState(ps.State):
	_channel: ps.Channel
	_auto_sync: bool
	_initial_selected: list[str]
	_initial_checked: list[str]
	_multiple: bool | None
	_on_node_expand_listener: ps.EventHandler1[str] | None
	_on_node_collapse_listener: ps.EventHandler1[str] | None

	def __init__(
		self,
		*,
		autoSync: bool = True,
		initialExpandedState: ExpandedState | None = None,
		# useTree options
		initialSelectedState: list[str] | None = None,
		initialCheckedState: list[str] | None = None,
		multiple: bool | None = None,
		onNodeExpand: ps.EventHandler1[str] | None = None,
		onNodeCollapse: ps.EventHandler1[str] | None = None,
	):
		self._channel = ps.channel()
		self._expanded: ExpandedState = dict(initialExpandedState or {})
		self._auto_sync = bool(autoSync)
		self._initial_selected = list(initialSelectedState or [])
		self._initial_checked = list(initialCheckedState or [])
		self._multiple = multiple
		self._on_node_expand_listener = onNodeExpand
		self._on_node_collapse_listener = onNodeCollapse
		# Client -> server per-node events
		self._channel.on("nodeExpand", self._on_node_expand)
		self._channel.on("nodeCollapse", self._on_node_collapse)

	# Public imperative API mirrors Mantine useTree
	def toggle_expanded(self, value: str):
		self._channel.emit("toggleExpanded", {"value": value})

	def expand(self, value: str):
		self._channel.emit("expand", {"value": value})

	def collapse(self, value: str):
		self._channel.emit("collapse", {"value": value})

	def expand_all_nodes(self):
		self._channel.emit("expandAllNodes")

	def collapse_all_nodes(self):
		self._channel.emit("collapseAllNodes")

	def set_expanded_state(self, state: ExpandedState):
		self._expanded.clear()
		self._expanded.update({k: bool(v) for k, v in state.items()})
		self._channel.emit("setExpandedState", {"expandedState": dict(self._expanded)})

	async def get_expanded_state(self) -> ExpandedState:
		result = await self._channel.request("getExpandedState")
		if isinstance(result, dict):
			# Update local cache with the result
			result = cast(dict[str, Any], result)
			self._expanded.clear()
			self._expanded.update({k: bool(v) for k, v in result.items()})
		return dict(self._expanded)

	# Selection API
	def toggle_selected(self, value: str):
		self._channel.emit("toggleSelected", {"value": value})

	def select(self, value: str):
		self._channel.emit("select", {"value": value})

	def deselect(self, value: str):
		self._channel.emit("deselect", {"value": value})

	def clear_selected(self):
		self._channel.emit("clearSelected")

	def set_selected_state(self, values: list[str]):
		self._channel.emit("setSelectedState", {"selectedState": list(values or [])})

	async def get_selected_state(self) -> list[str]:
		result = await self._channel.request("getSelectedState")
		return result if isinstance(result, list) else []

	async def get_anchor_node(self) -> str | None:
		result = await self._channel.request("getAnchorNode")
		return str(result) if isinstance(result, str) else None

	# Hover API
	def set_hovered_node(self, value: str | None):
		self._channel.emit("setHoveredNode", {"value": value})

	async def get_hovered_node(self) -> str | None:
		result = await self._channel.request("getHoveredNode")
		return str(result) if isinstance(result, str) else None

	# Checked API
	def check_node(self, value: str):
		self._channel.emit("checkNode", {"value": value})

	def uncheck_node(self, value: str):
		self._channel.emit("uncheckNode", {"value": value})

	def check_all_nodes(self):
		self._channel.emit("checkAllNodes")

	def uncheck_all_nodes(self):
		self._channel.emit("uncheckAllNodes")

	def set_checked_state(self, values: list[str]):
		self._channel.emit("setCheckedState", {"checkedState": list(values or [])})

	async def get_checked_nodes(self) -> list[dict[str, Any]]:
		result = await self._channel.request("getCheckedNodes")
		return result or []

	async def get_checked_state(self) -> list[str]:
		result = await self._channel.request("getCheckedState")
		return result if isinstance(result, list) else []

	async def is_node_checked(self, value: str) -> bool:
		result = await self._channel.request("isNodeChecked", {"value": value})
		return bool(result)

	async def is_node_indeterminate(self, value: str) -> bool:
		result = await self._channel.request("isNodeIndeterminate", {"value": value})
		return bool(result)

	@property
	def expanded_state(self) -> ExpandedState:
		return self._expanded

	# Client sync handlers
	async def _on_node_expand(self, payload: dict[str, Any]) -> None:
		if not isinstance(payload, dict):
			return
		value = payload.get("value")
		if isinstance(value, str) and value:
			self._expanded[value] = True
			listener = self._on_node_expand_listener
			if listener is not None:
				create_task(maybe_await(call_flexible(listener, value)))

	def _on_node_collapse(self, payload: dict[str, Any]) -> None:
		if not isinstance(payload, dict):
			return
		value = payload.get("value")
		if isinstance(value, str) and value:
			self._expanded[value] = False
			listener = self._on_node_collapse_listener
			if listener is not None:
				create_task(maybe_await(call_flexible(listener, value)))

	# Render the React wrapper component
	def render(
		self,
		*children: ps.Child,
		key: str | None = None,
		**props: Unpack[MantineTreeProps],
	):
		return TreeInternal(
			*children,
			key=key,
			channelId=self._channel.id,
			initialExpandedState=dict(self._expanded),
			autoSync=self._auto_sync,
			initialSelectedState=self._initial_selected,
			initialCheckedState=self._initial_checked,
			multiple=self._multiple,
			**props,
		)


def Tree(
	*children: ps.Child,
	key: str | None = None,
	state: TreeState | None = None,
	**props: Unpack[MantineTreeProps],
):
	if state is None:
		# No server state: render uncontrolled client Tree with no channel
		return TreeInternal(
			*children,
			key=key,
			channelId=None,
			# Let client use its own defaults for useTree; user can pass
			# Mantine props via **props.
			**props,
		)
	# With server state: delegate to state's render to include channel + options
	return state.render(*children, key=key, **props)



from typing import Literal

MantineFontFamily = Literal["monospace", "text", "heading"]
MantineTextAlign = Literal["left", "right", "center", "justify"]
MantineFontStyle = Literal["normal", "italic", "oblique"]
MantineTextTransform = Literal[
	"none",
	"capitalize",
	"uppercase",
	"lowercase",
	"full-width",
	"full-size-kana",
]
MantineTextDecoration = Literal["none", "underline", "overline", "line-through"]
MantinePosition = Literal["static", "relative", "absolute", "fixed", "sticky"]

DefaultMantineColor = (
	Literal[
		"dark",
		"gray",
		"red",
		"pink",
		"grape",
		"violet",
		"indigo",
		"blue",
		"cyan",
		"green",
		"lime",
		"yellow",
		"orange",
		"teal",
	]
	| str
)
MantineColor = str | DefaultMantineColor
MantineColorShade = Literal[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
MantineColorScheme = Literal["light", "dark", "auto"]

DefaultMantineSize = Literal["xs", "sm", "md", "lg", "xl"]
MantineSize = str | DefaultMantineSize

MantineBreakpoint = str | DefaultMantineSize
MantineBreakpointsValues = dict[MantineBreakpoint, str]
MantineFontSize = str  #
MantineFontSizesValues = dict[str, str]


MantineTextWrap = Literal["wrap", "nowrap", "balance", "pretty", "stable"]



from typing import Any

import pulse as ps


@ps.react_component("Input", "@mantine/core")
def Input(key: str | None = None, **props: Any): ...


@ps.react_component("Input", "@mantine/core", prop="Label")
def InputLabel(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Input", "@mantine/core", prop="Error")
def InputError(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Input", "@mantine/core", prop="Description")
def InputDescription(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Input", "@mantine/core", prop="Placeholder")
def InputPlaceholder(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Input", "@mantine/core", prop="Wrapper")
def InputWrapper(*children: ps.Child, key: str | None = None, **props: Any): ...


@ps.react_component("Input", "@mantine/core", prop="ClearButton")
def InputClearButton(key: str | None = None, **props: Any): ...



import json
from collections.abc import Iterator, Mapping, Sequence
from datetime import datetime
from typing import (
	Any,
	Generic,
	Literal,
	TypeVar,
	Unpack,
	cast,
)

import pulse as ps
from pulse.helpers import call_flexible, create_task, maybe_await
from pulse.reactive_extensions import ReactiveDict
from pulse.serializer import deserialize

from .internal import FormInternal, FormMode
from .validators import (
	AsyncValidator,
	ServerValidation,
	Validation,
	Validator,
	serialize_validation,
)

FieldValue = str | int | float | bool | datetime | ps.UploadFile
FormValues = Mapping[str, "FieldValue | Sequence[FieldValue] | FormValues"]

TForm = TypeVar("TForm", bound=FormValues)


class MantineFormProps(ps.HTMLFormProps, Generic[TForm], total=False):
	mode: FormMode
	validate: "Validation"
	initialValues: dict[str, Any]
	initialErrors: dict[str, Any]
	initialDirty: dict[str, bool]
	initialTouched: dict[str, bool]
	validateInputOnBlur: bool | list[str]
	validateInputOnChange: bool | list[str]
	clearInputErrorOnChange: bool
	cascadeUpdates: bool
	debounceMs: int
	touchTrigger: Literal["change", "focus"]
	"""touchTrigger option allows customizing events that change touched state.

    Options:
        change (default): Field is touched when value changes or has been focused
        focus: Field is touched only when it has been focused
    """
	onSubmit: ps.EventHandler1[TForm]  # pyright: ignore[reportIncompatibleVariableOverride]
	onReset: ps.EventHandler1[ps.FormEvent[ps.HTMLFormElement]]
	syncMode: Literal["none", "blur", "change"]


class MantineForm(ps.State, Generic[TForm]):
	_channel: ps.Channel
	_form: ps.ManualForm
	_synced_values: ReactiveDict[str, Any]
	_validation: "Validation | None"
	_mantine_props: dict[str, Any]
	_on_submit: ps.EventHandler1[TForm] | None

	def __init__(
		self,
		mode: FormMode | None = None,
		validate: "Validation | None" = None,
		initialValues: dict[str, Any] | None = None,
		initialErrors: dict[str, Any] | None = None,
		initialDirty: dict[str, bool] | None = None,
		initialTouched: dict[str, bool] | None = None,
		validateInputOnBlur: bool | list[str] | None = None,
		validateInputOnChange: bool | list[str] | None = None,
		clearInputErrorOnChange: bool | None = None,
		touchTrigger: Literal["change", "focus"] | None = None,
		syncMode: Literal["none", "blur", "change"] = "none",
		debounceMs: int | None = None,
	):
		self._channel = ps.channel()
		self._form = ps.ManualForm(on_submit=self._handle_form_data)
		self._sync_mode: Literal["none", "blur", "change"] = syncMode
		self._synced_values = ReactiveDict(initialValues or {})
		if self._sync_mode != "none":
			self._channel.on("syncValues", self._on_sync_values)
		# Listen for server-side validation requests from the client
		self._channel.on("serverValidate", self._on_server_validate)

		self._validation = validate
		self._mantine_props = {
			"mode": mode,
			"validate": serialize_validation(validate) if validate else None,
			"initialValues": initialValues,
			"initialErrors": initialErrors,
			"initialDirty": initialDirty,
			"initialTouched": initialTouched,
			"validateInputOnBlur": validateInputOnBlur,
			"validateInputOnChange": validateInputOnChange,
			"clearInputErrorOnChange": clearInputErrorOnChange,
			"debounceMs": debounceMs,
			"touchTrigger": touchTrigger,
			"syncMode": syncMode,
		}
		# Filter out None values
		self._mantine_props = {
			k: v for k, v in self._mantine_props.items() if v is not None
		}

		_check_for_reserved_keys(initialValues)
		_check_for_reserved_keys(initialErrors)
		_check_for_reserved_keys(initialDirty)
		_check_for_reserved_keys(initialTouched)

	async def _handle_form_data(self, data: ps.FormData):
		# Expect one JSON-serialized entry under "__data__" with v3 serializer
		# and remaining entries are files keyed by their dot/bracket paths.
		raw = data.get("__data__")
		base: dict[str, Any] = {}
		if isinstance(raw, str) and raw:
			try:
				payload = json.loads(raw)
				base = deserialize(payload)
			except Exception:
				base = {}

		# Merge file entries back into the nested structure
		files: dict[str, Any] = {k: v for k, v in data.items() if k != "__data__"}
		result = _merge_files_into_structure(base, files)

		# Run server-side validation for ALL rules before forwarding to user's onSubmit.
		# This mirrors Mantine behavior where onSubmit is called only if the form is valid.
		if not await self._validate_all_before_submit(result):
			return

		# Forward to user onSubmit if provided
		if self._on_submit is not None:
			await maybe_await(call_flexible(self._on_submit, result))

	# Mount the React component, wiring messages and passing through props
	def render(
		self,
		*children: ps.Child,
		key: str | None = None,
		onSubmit: ps.EventHandler1[TForm] | None = None,
		**props: Unpack[ps.HTMLFormProps],  # pyright: ignore[reportGeneralTypeIssues]
	):
		self._on_submit = onSubmit
		merged: dict[str, Any] = {**props, **self._mantine_props, **self._form.props()}
		return FormInternal(
			*children,
			key=key,
			channelId=self._channel.id,
			**merged,
		)

	# Public API mapping to Mantine useForm actions
	async def get_form_values(self):
		return await self._channel.request("getFormValues")

	def set_values(self, values: dict[str, Any]):
		# Optimistically update server state if sync is enabled
		if self._sync_mode != "none" and isinstance(values, dict):
			incoming_keys = set(values.keys())
			for existing in list(self._synced_values.keys()):
				if existing not in incoming_keys:
					self._synced_values.delete(existing)
			self._synced_values.update(values)
		self._channel.emit("setValues", {"values": values})

	def set_field_value(self, path: str, value: Any):
		# Optimistically update server state if sync is enabled
		if self._sync_mode != "none" and isinstance(path, str):
			try:
				segments = _tokenize_path(path)
				_set_deep(self._synced_values, segments, value)
			except Exception:
				pass
		self._channel.emit("setFieldValue", {"path": path, "value": value})

	def insert_list_item(self, path: str, item: Any, index: int | None = None):
		msg: dict[str, Any] = {"path": path, "item": item}
		if index is not None:
			msg["index"] = index
		# Optimistically update server state if sync is enabled
		if self._sync_mode != "none" and isinstance(path, str):
			try:
				segments = _tokenize_path(path)
				parent, key = _get_parent_and_key(self._synced_values, segments)
				if isinstance(key, int) and isinstance(parent, list):
					parent = cast(list[Any], parent)
					insert_at = index if index is not None else len(parent)
					if insert_at < 0:
						insert_at = 0
					if insert_at > len(parent):
						insert_at = len(parent)
					parent.insert(insert_at, item)
				elif isinstance(key, str) and isinstance(parent, dict):
					# If the target is a list under a dict key
					lst = parent.get(key)
					if isinstance(lst, list):
						lst = cast(list[Any], lst)
						insert_at = index if index is not None else len(lst)
						if insert_at < 0:
							insert_at = 0
						if insert_at > len(lst):
							insert_at = len(lst)
						lst.insert(insert_at, item)
			except Exception:
				pass
		self._channel.emit("insertListItem", msg)

	def remove_list_item(self, path: str, index: int):
		# Optimistically update server state if sync is enabled
		if self._sync_mode != "none" and isinstance(path, str):
			try:
				segments = _tokenize_path(path)
				parent, key = _get_parent_and_key(self._synced_values, segments)
				lst: list[Any] | None = None
				if isinstance(key, int) and isinstance(parent, list):
					lst = parent
				elif isinstance(key, str) and isinstance(parent, dict):
					maybe = parent.get(key)
					if isinstance(maybe, list):
						lst = maybe
				if lst is not None and 0 <= index < len(lst):
					lst.pop(index)
			except Exception:
				pass
		self._channel.emit("removeListItem", {"path": path, "index": index})

	def reorder_list_item(self, path: str, frm: int, to: int):
		# Optimistically update server state if sync is enabled
		if self._sync_mode != "none" and isinstance(path, str):
			try:
				segments = _tokenize_path(path)
				parent, key = _get_parent_and_key(self._synced_values, segments)
				lst: list[Any] | None = None
				if isinstance(key, int) and isinstance(parent, list):
					lst = parent
				elif isinstance(key, str) and isinstance(parent, dict):
					maybe = parent.get(key)
					if isinstance(maybe, list):
						lst = maybe
				if lst is not None and 0 <= frm < len(lst) and 0 <= to < len(lst):
					item = lst.pop(frm)
					lst.insert(to, item)
			except Exception:
				pass
		self._channel.emit(
			"reorderListItem",
			{"path": path, "from": frm, "to": to},
		)

	def set_errors(self, errors: dict[str, Any]):
		self._channel.emit("setErrors", {"errors": errors})

	def set_field_error(self, path: str, error: Any):
		self._channel.emit("setFieldError", {"path": path, "error": error})

	def clear_errors(self, *paths: str):
		if paths:
			self._channel.emit("clearErrors", {"paths": list(paths)})
		else:
			self._channel.emit("clearErrors")

	def set_touched(self, touched: dict[str, bool]):
		self._channel.emit("setTouched", {"touched": touched})

	def validate(self):
		# Trigger client-side validation
		self._channel.emit("validate")
		# Also run all server-side validators in the background for current values
		create_task(self._validate_all_server_specs())

	async def _validate_all_server_specs(self) -> None:
		try:
			# Get latest values from client
			values: dict[str, Any] = await self._channel.request("getFormValues")
			# Collect all paths that have at least one ServerValidation spec
			schema = self._validation
			if not isinstance(schema, dict):
				return

			paths: list[str] = []

			def join(p: str, k: str) -> str:
				return f"{p}.{k}" if p else k

			def visit(node: Any, path: str) -> None:
				if isinstance(node, dict):
					root = node.get("formRootRule")
					if root is not None:
						visit(root, path)
					for k, v in node.items():
						if not isinstance(k, str):
							continue
						if k == "formRootRule":
							continue
						visit(v, join(path, k))
					return
				if isinstance(node, list):
					has_server = any(
						isinstance(spec, ServerValidation) for spec in node
					)
					if has_server:
						paths.append(path)
					return
				if isinstance(node, ServerValidation):
					paths.append(path)

			visit(schema, "")

			for path in paths:
				payload = {
					"value": None,  # value will be recomputed in handler using values + path
					"values": values,
					"path": path,
				}
				await self._on_server_validate(payload)
		except Exception:
			return

	def reset(self, initial_values: dict[str, Any] | None = None):
		if initial_values is not None:
			self._channel.emit("reset", {"initialValues": initial_values})
		else:
			self._channel.emit("reset")

	@property
	def values(self) -> ReactiveDict[str, Any]:
		if self._sync_mode == "none":
			raise ValueError(
				'Form values are only accessible on the server when syncMode="change" or syncMode="blur"'
			)
		return self._synced_values

	# --- internal sync handling -------------------------------------------------
	def _on_sync_values(self, payload: dict[str, Any]) -> None:
		values = payload.get("values")
		if not isinstance(values, dict):
			return
		values = cast(dict[str, Any], values)
		incoming_keys = set(values.keys())
		for existing in list(self._synced_values.keys()):
			if existing not in incoming_keys:
				self._synced_values.delete(existing)
		self._synced_values.update(values)

	# Channel handler for server validation (single entrypoint)
	async def _on_server_validate(self, payload: dict[str, Any]) -> None:
		try:
			value = payload.get("value")
			values = payload.get("values")
			path = payload.get("path")
			if not isinstance(values, dict) or not isinstance(path, str):
				return

			values = cast(dict[str, Any], values)

			schema = self._validation
			if not isinstance(schema, dict):
				return

			# Traverse schema by path segments, skipping numeric indices
			node: Any = schema
			for seg in str(path).split("."):
				if isinstance(node, dict):
					# Skip numeric segments (list indices)
					if seg.isdigit():
						continue
					nxt = node.get(seg)
					if nxt is None:
						# Try root-level rule fallback
						nxt = node.get("formRootRule")
					node = nxt
				else:
					break

			# Node can be a spec, a list of specs, or nested dict
			if isinstance(node, dict):
				node = node.get("formRootRule")
			specs: list[Validator] = []
			if isinstance(node, list):
				for candidate in node:
					if isinstance(candidate, Validator):
						specs.append(candidate)
			elif isinstance(node, Validator):
				specs.append(node)

			# Invoke server validators, stop on first error
			for spec in specs:
				if isinstance(spec, ServerValidation):
					try:
						res = await spec.acheck(value, values, path)
						if isinstance(res, str) and res:
							self.set_field_error(path, res)
							return
					except Exception:
						# Do not crash validation on server errors; surface generic error
						self.set_field_error(path, "Validation failed")
						return
			self.clear_errors(path)
		except Exception:
			# best-effort; do not crash channel
			return

	# Validate all rules during submit (client and server equivalents)
	async def _validate_all_before_submit(self, values: dict[str, Any]) -> bool:
		schema = self._validation
		if not isinstance(schema, dict):
			return True

		errors: dict[str, Any] = {}

		def join(p: str, k: str) -> str:
			return f"{p}.{k}" if p else k

		def get_value_at_path(source: Any, path: str) -> Any:
			cur = source
			for seg in str(path).split("."):
				if seg == "":
					continue
				if isinstance(cur, list) and seg.isdigit():
					cur = cast(list[Any], cur)
					idx = int(seg)
					if idx < 0 or idx >= len(cur):
						return None
					cur = cur[idx]
				elif isinstance(cur, dict):
					cur = cast(dict[str, Any], cur)
					cur = cur.get(seg)
				else:
					return None
			return cur

		async def apply_node(node: Any, path: str) -> None:
			# If nested dict contains root-level rule, apply it to current path
			if isinstance(node, dict):
				root = node.get("formRootRule")
				if root is not None:
					await apply_node(root, path)
				for k, v in node.items():
					if not isinstance(k, str):
						continue
					if k == "formRootRule":
						continue
					await apply_node(v, join(path, k))
				return
			# List of specs
			if isinstance(node, list):
				for spec in node:
					await apply_node(spec, path)
				return
			# Single validator
			if isinstance(node, Validator):
				try:
					value = get_value_at_path(values, path)
					# Prefer async if available
					if isinstance(node, AsyncValidator):
						res = await node.acheck(value, values, path)
					else:
						res = node.check(value, values, path)
					if isinstance(res, str) and res:
						errors[path] = res
				except Exception:
					errors[path] = "Validation failed"
				return

		await apply_node(schema, "")

		if errors:
			self.set_errors(errors)
			return False
		return True


# Also ensure user data objects do not contain reserved keys
def _check_for_reserved_keys(obj: Any, path: str = "") -> None:
	if isinstance(obj, dict):
		for k, v in obj.items():
			if not isinstance(k, str):
				continue
			if k in {"$kind", "formRootRule"}:
				raise ValueError(
					"'$kind' and 'formRootRule' are reserved keys and cannot appear in user data"
				)
			_check_for_reserved_keys(v, f"{path}.{k}" if path else str(k))
	elif isinstance(obj, list):
		for idx, v in enumerate(cast(list[Any], obj)):
			_check_for_reserved_keys(v, f"{path}[{idx}]")


def _merge_files_into_structure(base: Any, files: dict[str, Any]) -> Any:
	result = _deep_copy(base)

	# Expand lists of files into multiple insert operations
	def iter_entries() -> Iterator[tuple[str, Any]]:
		for path, value in files.items():
			if isinstance(value, list):
				value = cast(list[Any], value)
				for item in value:
					yield (path, item)
			else:
				yield (path, value)

	for path, value in iter_entries():
		segments = _tokenize_path(path)
		_set_deep(result, segments, value)
	return result


def _deep_copy(obj: Any) -> Any:
	if isinstance(obj, dict):
		return {k: _deep_copy(v) for k, v in obj.items()}
	if isinstance(obj, list):
		return [_deep_copy(v) for v in obj]
	return obj


def _tokenize_path(path: str) -> list[str]:
	# Convert bracket notation to dots: a[0].b -> a.0.b
	out: list[str] = []
	buf = ""
	i = 0
	while i < len(path):
		ch = path[i]
		if ch == "[":
			if buf:
				out.append(buf)
				buf = ""
			j = i + 1
			num = ""
			while j < len(path) and path[j] != "]":
				num += path[j]
				j += 1
			if num:
				out.append(num)
			i = j + 1
			if i < len(path) and path[i] == ".":
				i += 1
			continue
		elif ch == ".":
			if buf:
				out.append(buf)
				buf = ""
		else:
			buf += ch
		i += 1
	if buf:
		out.append(buf)
	return [seg for seg in out if seg]


def _ensure_container(parent: Any, key: str | int, next_is_index: bool) -> Any:
	if isinstance(key, int):
		# Parent must be a list
		if not isinstance(parent, list):
			return []
		# Ensure capacity
		parent = cast(list[Any], parent)
		while len(parent) <= key:
			parent.append(None)
		child = parent[key]
		if child is None:
			parent[key] = [] if next_is_index else {}
			child = parent[key]
		return child
	else:
		if not isinstance(parent, dict):
			return {}
		parent = cast(dict[str, Any], parent)
		child = parent.get(key)
		if child is None:
			parent[key] = [] if next_is_index else {}
			child = parent[key]
		return child


def _set_deep(root: Any, segments: list[str], value: Any) -> None:
	# Walk creating containers as needed; set or append at leaf
	cur = root
	for idx, raw_seg in enumerate(segments):
		is_last = idx == len(segments) - 1
		is_index = raw_seg.isdigit()
		seg: int | str = int(raw_seg) if is_index else raw_seg
		if is_last:
			if isinstance(seg, int):
				if not isinstance(cur, list):
					return
				cur = cast(list[Any], cur)
				while len(cur) <= seg:
					cur.append(None)
				existing = cur[seg]
				if existing is None:
					cur[seg] = value
				else:
					if isinstance(existing, list):
						existing = cast(list[Any], existing)
						existing.append(value)
					else:
						cur[seg] = [existing, value]
			else:
				if not isinstance(cur, dict):
					return
				cur = cast(dict[str, Any], cur)
				existing = cur.get(seg)
				if existing is None:
					cur[seg] = value
				else:
					if isinstance(existing, list):
						existing = cast(list[Any], existing)
						existing.append(value)
					else:
						cur[seg] = [existing, value]
		else:
			next_is_index = segments[idx + 1].isdigit()
			child = _ensure_container(cur, seg, next_is_index)
			# Attach child back to parent in case container was created
			if isinstance(seg, int):
				if not isinstance(cur, list):
					return
				cur = cast(list[Any], cur)
				while len(cur) <= seg:
					cur.append(None)
				cur[seg] = child
			else:
				if not isinstance(cur, dict):
					return
				cur = cast(dict[str, Any], cur)
				cur[seg] = child
			cur = child


def _walk_to_parent(root: Any, segments: list[str]) -> tuple[Any, str | int]:
	cur = root
	for idx, raw_seg in enumerate(segments):
		is_last = idx == len(segments) - 1
		is_index = raw_seg.isdigit()
		seg: int | str = int(raw_seg) if is_index else raw_seg
		if is_last:
			return cur, seg
		next_is_index = segments[idx + 1].isdigit()
		child = _ensure_container(cur, seg, next_is_index)
		if isinstance(seg, int):
			if not isinstance(cur, list):
				return cur, seg
			cur = cast(list[Any], cur)
			while len(cur) <= seg:
				cur.append(None)
			cur[seg] = child
		else:
			if not isinstance(cur, dict):
				return cur, seg
			cur = cast(dict[str, Any], cur)
			cur[seg] = child
		cur = child
	return cur, segments[-1] if segments else ""


def _get_parent_and_key(root: Any, segments: list[str]) -> tuple[Any, str | int]:
	return _walk_to_parent(root, segments)



from __future__ import annotations

from typing import Any, Literal, TypedDict, Unpack

import pulse as ps
from pulse.codegen.imports import ImportStatement

from .styles import CSSVariables
from .theme import MantineTheme, MantineThemeOverride
from .types import MantineColorScheme

MantineProviderForceColorScheme = Literal["light", "dark"]
MantineProviderEnvironment = Literal["default", "test"]


class MantineProviderProps(TypedDict, total=False):
	"""Theme override object"""

	theme: MantineThemeOverride
	colorSchemeManager: MantineColorSchemeManager
	"""Used to retrieve/set color scheme value in external storage, by default uses `window.localStorage`"""
	defaultColorScheme: MantineColorScheme
	"""Default color scheme value used when `colorSchemeManager` cannot retrieve value from external storage, `light` by default"""
	forceColorScheme: MantineProviderForceColorScheme
	"""Forces color scheme value, if set, MantineProvider ignores `colorSchemeManager` and `defaultColorScheme`"""
	cssVariablesSelector: str
	"""CSS selector to which CSS variables should be added, `:root` by default"""
	withCssVariables: bool
	"""Determines whether theme CSS variables should be added to given `cssVariablesSelector` @default `true`"""
	deduplicateCssVariables: bool
	"""Determines whether CSS variables should be deduplicated: if CSS variable has the same value as in default theme, it is not added in the runtime. @default `true`."""
	getRootElement: ps.JsFunction[[], ps.HTMLElement | None]
	"""Function to resolve root element to set `data-mantine-color-scheme` attribute, must return undefined on server, `() => document.documentElement` by default"""
	classNamesPrefix: str
	"""A prefix for components static classes (for example {selector}-Text-root), `mantine` by default"""
	getStyleNonce: ps.JsFunction[[], str]
	"""Function to generate nonce attribute added to all generated `<style />` tags"""
	cssVariablesResolver: CSSVariablesResolver
	"""Function to generate CSS variables based on theme object"""
	withStaticClasses: bool
	"""Determines whether components should have static classes, for example, `mantine-Button-root`. @default `true`"""
	withGlobalClasses: bool
	"""Determines whether global classes should be added with `<style />` tag. Global classes are required for `hiddenFrom`/`visibleFrom` and `lightHidden`/`darkHidden` props to work. @default `true`."""
	stylesTransform: MantineStylesTransform
	"""An object to transform `styles` and `sx` props into css classes, can be used with CSS-in-JS libraries"""
	env: MantineProviderEnvironment
	"""Environment at which the provider is used, `'test'` environment disables all transitions and portals"""


@ps.react_component(
	"MantineProvider",
	"@mantine/core",
	extra_imports=[
		ImportStatement(
			src="@mantine/core/styles.css",
			side_effect=True,
			# Ensure core styles load before dates/charts styles if present
			before=["@mantine/dates/styles.css", "@mantine/charts/styles.css"],
		)
	],
)
def MantineProvider(*children: ps.Child, key: str | None = None, **props: Any): ...


class MantineStylesTransform(TypedDict, total=False):
	sx: ps.JsFunction[[], ps.JsFunction[[Any], str]]
	styles: ps.JsFunction[[], ps.JsFunction[[Any, Any], dict[str, str]]]


class ConvertCSSVariablesInput(TypedDict):
	variables: CSSVariables
	"Shared CSS variables that should be accessible independent from color scheme"
	dark: CSSVariables
	"CSS variables available only in dark color scheme"
	light: CSSVariables
	"CSS variables available only in light color scheme"


CSSVariablesResolver = ps.JsFunction[[MantineTheme], ConvertCSSVariablesInput]


class MantineColorSchemeManager(TypedDict):
	get: ps.JsFunction[[MantineColorScheme], MantineColorScheme]
	"""Function to retrieve color scheme value from external storage, for example window.localStorage
    
    JS: get: (defaultValue: MantineColorScheme) => MantineColorScheme"""

	set: ps.JsFunction[[MantineColorScheme], None]
	"""Function to set color scheme value in external storage, for example window.localStorage
    
    JS: set: (value: MantineColorScheme) => void"""

	subscribe: ps.JsFunction[[ps.JsFunction[[MantineColorScheme], None]], None]
	"""Function to subscribe to color scheme changes triggered by external events
    
    JS: subscribe: (onUpdate: (colorScheme: MantineColorScheme) => void) => void"""

	unsubscribe: ps.JsFunction[[], None]
	"""Function to unsubscribe from color scheme changes triggered by external events
    
    JS: unsubscribe: () => void"""

	clear: ps.JsFunction[[], None]
	"""Function to clear value from external storage
    
    JS: clear: () => void"""


class HeadlessMantineProviderProps(TypedDict, total=False):
	theme: MantineThemeOverride
	"""Theme override object"""

	env: MantineProviderEnvironment
	"""Environment at which the provider is used, 'test' environment disables all transitions and portals"""


@ps.react_component("HeadlessMantineProvider", "@mantine/core")
def HeadlessMantineProvider(
	*children: ps.Child,
	key: str | None = None,
	**props: Unpack[HeadlessMantineProviderProps],
): ...
